@@ -2,22 +2,27 @@
 ### LU decomposition and singular subsamples handling
 require(robustbase)
 source(system.file("xtraR/subsample-fns.R", package = "robustbase", mustWork=TRUE))
+source(system.file("test-tools-1.R", package="Matrix", mustWork=TRUE))
+require(Matrix)
+
+cat("doExtras:", doExtras <- robustbase:::doExtras(),"\n")
+showProc.time()
 
 A <- matrix(c(0.001, 1, 1, 2), 2)
 set.seed(11)
-str(sa <- subsample(A))
+str(sa <- tstSubsample(A))
 
 A <- matrix(c(3, 2, 6, 17, 4, 18, 10, -2, 12), 3)
-subsample(A)
+tstSubsample(A)
 
 ## test some random matrix
 set.seed(1002)
 A <- matrix(rnorm(100), 10)
-subsample(A)
+tstSubsample(A)
 
 ## test singular matrix handling
 A <- matrix(c(1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 1), 4, byrow=TRUE)
-subsample(A)
+tstSubsample(A)
 
 
 ## test subsample with mts > 0
@@ -25,7 +30,7 @@ data <- data.frame(y = rnorm(9), expand.grid(A = letters[1:3], B = letters[1:3])
 x <- model.matrix(y ~ ., data)
 y <- data$y
 ## this should produce a warning and return status == 2
-z <- Rsubsample(x, y, mts=2)
+showSys.time(z <- Rsubsample(x, y, mts=2))
 stopifnot(z$status == 2)
 
 
@@ -33,17 +38,18 @@ stopifnot(z$status == 2)
 ## columns only
 X <- matrix(c(1e-7, 2, 1e-10, 0.2), 2)
 y <- 1:2
-subsample(t(X), y)
+tstSubsample(t(X), y)
 
 ## rows only
 X <- matrix(c(1e-7, 2, 1e10, 0.2), 2)
 y <- 1:2
-subsample(X, y)
+tstSubsample(X, y)
 
 ## both
 X <- matrix(c(1e-7, 1e10, 2, 2e12), 2)
 y <- 1:2
-subsample(X, y)
+tstSubsample(X, y)
+showProc.time()
 
 
 ## test real data example
@@ -56,43 +62,27 @@ y <- model.response(mf)
 stopifnot(qr(X)$rank == ncol(X))
 
 ## this used to fail: different pivots in step 37
-str(s1 <- subsample(X, y))
-s2 <- subsample(X / max(abs(X)), y / max(abs(X)))
-s3 <- subsample(X * 2^-50, y * 2^-50)
+str(s1 <- tstSubsample(X, y))
+s2 <- tstSubsample(X / max(abs(X)), y / max(abs(X)))
+s3 <- tstSubsample(X * 2^-50, y * 2^-50)
 ## all components *BUT*  x, y, lu, Dr, Dc, rowequ, colequ :
 nm <- names(s1); nm <- nm[is.na(match(nm, c("x","y","lu", "Dr", "Dc", "rowequ", "colequ")))]
-stopifnot(all.equal(s1[nm], s2[nm], tol=1e-10),
-	  all.equal(s1[nm], s3[nm], tol=1e-10))
-
-## test subsampling
-testSubSampling <- function(X, y) {
-    lX <- X[sample(nrow(X)), ]
-    ## C version
-    zc <- Rsubsample(lX, y)
-    ## R version
-    zR <- LU.gaxpy(t(lX))
-    if (as.logical(zc$status)) {
-        ## singularity in C detected
-        if (!zR$singular)
-            stop("singularity in C but not in R")
-    } else {
-        ## no singularity detected
-        if (zR$singular)
-            stop("singularity in R but not in C")
-    }
-    zR$singular
-}
+stopifnot(all.equal(s1[nm], s2[nm], tolerance=1e-10),
+	  all.equal(s1[nm], s3[nm], tolerance=1e-10))
+showProc.time()
 
 set.seed(10)
-nsing <- sum(replicate(200, testSubSampling(X, y)))
+nsing <- sum(replicate(if(doExtras) 200 else 20, tstSubsampleSing(X, y)))
 stopifnot(nsing == 0)
+showProc.time()
 
 ## test example with many categorical predictors
 set.seed(10)
 r1 <- lmrob(Diversity ~ .^2 , data = possumDiv, cov="none")
 ## lmrob.S used to fail for this seed:
 set.seed(108)
-lmrob(Diversity ~ .^2 , data = possumDiv, cov="none") #, trace=4)
+r2 <- lmrob(Diversity ~ .^2 , data = possumDiv, cov="none") #, trace=4)
+showProc.time()
 
 ## investigate problematic subsample:
 idc <- 1 + c(140, 60, 12, 13, 89, 90, 118, 80, 17, 134, 59, 94, 36,
@@ -105,7 +95,7 @@ rc <- lm(Diversity ~ .^2 , data = possumDiv, subset = idc)
 
 X <- model.matrix(rc)
 y <- possumDiv$Diversity[idc]
-subsample(X, y)
+tstSubsample(X, y)## have different pivots ... could not find non-singular
 
 lu <- LU.gaxpy(t(X))
 stopifnot(lu$sing)
@@ -114,26 +104,26 @@ stopifnot(zc$status > 0)
 ## column 52 is linearly dependent and should have been discarded
 ## qr(t(X))$pivot
 
-image(as(round(zc$lu - (lu$L + lu$U - diag(nrow(lu$U))), 10), "Matrix"))
-image(as(sign(zc$lu) - sign(lu$L + lu$U - diag(nrow(lu$U))), "Matrix"))
-
+image(as(round(zc$lu -      (lu$L + lu$U - diag(nrow(lu$U))), 10), "Matrix"))
+image(as( sign(zc$lu) - sign(lu$L + lu$U - diag(nrow(lu$U))),      "Matrix"))
+showProc.time()
 
 ## test equilibration
 ## colequ only
 X <- matrix(c(1e-7, 2, 1e-10, 0.2), 2)
 y <- 1:2
-subsample(t(X), y)
+tstSubsample(t(X), y)
 
 ## rowequ only
 X <- matrix(c(1e-7, 2, 1e10, 0.2), 2)
 y <- 1:2
-subsample(X, y)
+tstSubsample(X, y)
 
 ## both
 X <- matrix(c(1e-7, 1e10, 2, 2e12), 2)
 y <- 1:2
-subsample(X, y)
-
+tstSubsample(X, y)
+showProc.time()
 
 ### real data, see MM's ~/R/MM/Pkg-ex/robustbase/hedlmrob.R
 ##  close to singular cov():
@@ -144,6 +134,9 @@ fm1 <- lmrob(y ~ a + I(a^2) + tf + I(tf^2) + A + I(A^2) + . , data = d1k27)
 ## --> cov = ".vcov.w"
 fm2 <- lmrob(y ~ a + I(a^2) + tf + I(tf^2) + A + I(A^2) + . , data = d1k27,
              cov = ".vcov.w", trace = TRUE)
+showProc.time()# 2.77
+
+if(doExtras) {##-----------------------------------------------------------------
 
 ## Q: does it change to use numeric instead of binary factors ?
 ## A: not really ..
@@ -154,13 +147,14 @@ fm1.n <- lmrob(y ~ a + I(a^2) + tf + I(tf^2) + A + I(A^2) + . , data = d1k.n)
 fm2.n <- lmrob(y ~ a + I(a^2) + tf + I(tf^2) + A + I(A^2) + . , data = d1k.n,
              cov = ".vcov.w", trace = 2)
 
-summary(weights(fm1, type="robustness"))
+print(summary(weights(fm1, type="robustness")))
 hist(weights(fm1, type="robustness"), main="robustness weights of fm1")
 rug(weights(fm1, type="robustness"))
+showProc.time()## 2.88
 
 ##
 fmc <- lm   (y ~ poly(a,2)-a + poly(tf, 2)-tf + poly(A, 2)-A + . , data = d1k27)
-summary(fmc)
+print(summary(fmc))
 ## -> has NA's for  'a, tf, A'  --- bad that it did *not* work to remove them
 
 nform <- update(formula(fm1), ~ .
@@ -172,8 +166,9 @@ fm1. <- lmrob(nform, data = d1k27)# now w/o warning !? !!
 fm2. <- lmrob(nform, data = d1k27, cov = ".vcov.w", trace = TRUE)
 
 ## now lmrob takes care of NA coefficients automatically
-lmrob(y ~ poly(a,2)-a + poly(tf, 2)-tf + poly(A, 2)-A + . , data = d1k27)
-
+print(lmrob(y ~ poly(a,2)-a + poly(tf, 2)-tf + poly(A, 2)-A + . , data = d1k27))
+showProc.time() ## 4.24
+} ## only if(doExtras) ##--------------------------------------------------------
 
 ## test exact fit property
 set.seed(20)
@@ -185,4 +180,5 @@ lmrob.S(x, data$y, lmrob.control())
 summary(ret)
 
 
-cat('Time elapsed: ', proc.time(),'\n') # for ``statistical reasons''
+showProc.time()
+
